/*
**   IRC - Internet Relay Chat, source/fdlist.c
**   Copyright (C) 1999 Chris Behrens
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ircd_includes.h"
#include "fdlist.h"

RCSID("$Id: fdlist.c,v 1.6 1999/02/03 01:14:07 cbehrens Exp $");

extern struct kevent *kqchange;
extern int kqchangesize;
extern int kqchangepos;

struct kevent *fdlist_kqueue_increase(struct kevent *kq, int kqsize)
{
	return (struct kevent *)MyRealloc(kq, kqsize * sizeof(struct kevent));
}

static int _fdlist_event(IRCD_CONN *cptr, int flags, int filter)
{
	struct kevent *kv;

	if (kqchangesize < kqchangepos + 64)
	{
		kqchangesize += 64;
		if ((kqchange = fdlist_kqueue_increase(kqchange, kqchangesize)) == NULL)
			return -ENOMEM;
	}

	kv = kqchange + kqchangepos++;

	kv->ident = cptr->fd;
	kv->flags = flags;
	kv->filter = filter;
	kv->fflags = 0;
	kv->data = NULL;
	kv->udata = cptr;

	return 0;
}

int fdlist_add_event(IRCD_CONN *cptr, int flags)
{
        if (flags & FDLIST_FLAGS_READ)
		_fdlist_event(cptr, EV_ADD, EVFILT_READ);
                
        if (flags & FDLIST_FLAGS_WRITE)
		_fdlist_event(cptr, EV_ADD, EVFILT_WRITE);

	return 0;
}

int fdlist_del_event(IRCD_CONN *cptr, int flags)
{
        if (flags & FDLIST_FLAGS_READ)
		_fdlist_event(cptr, EV_DELETE, EVFILT_READ);
                
        if (flags & FDLIST_FLAGS_WRITE)
		_fdlist_event(cptr, EV_DELETE, EVFILT_WRITE);

	return 0;
}
